#!/bin/bash

# this script ensures that there is private key (formats: PKCS1 and PKCS8)
# in case a new key has to be generated a CSR will be created as well.
# the CSR is downloaded by nevisAdmin 4 after phase PREPARE and sent to the internal CA.
# also the CSR is stored in the nevisAdmin 4 DB so that it can later be downloaded by the user
# and sent to an external CA for issuing an official certificate.
# the remainder of the keystore is established in phase CONFIGURE by pki_complete_keystore.sh

if [[ $# -ne 2 ]] ; then
    echo ""
    echo "this script is deployed by nevisAdmin 4 and will be executed for each key store in phase PREPARE"
    echo ""
    echo 'arguments: <keyStoreName> <dn>'
    echo ''
    echo 'example: ./pki_prepare_keystore.sh proxy "/O=SIVEN/CN=www.siven.ch"'
    echo ''
    exit -1
fi

# ensure output directory exists
mkdir -p /tmp/nevisAdmin4/csr

name=$1
dn=$2

path="/var/opt/keys/own/${name}"

# ensure path exists
mkdir -p ${path}

renew_csr=0

if [[ ! -f "${path}/key_pkcs1.pem" ]]; then
  # PKCS1 is our master
  echo "generate PKCS1 private key ($path/key_pkcs1.pem)"
  openssl genrsa > ${path}/key_pkcs1.pem 2>/dev/null
  # if the master changes we have to recreate the CSR
  renew_csr=1
fi

if [[ ! -f "${path}/key_pkcs8.pem" ]]; then
  # filebeats requires PKCS8 format
  echo "generate PKCS8 private key ($path/key_pkcs8.pem)"
  openssl pkcs8 -topk8 -inform pem -in ${path}/key_pkcs1.pem -outform PEM -nocrypt -out ${path}/key_pkcs8.pem
fi

if [[ ! -f "${path}/key_pub.pem" ]]; then
  echo "exporting public key ($path/key_pub.pem)"
  openssl rsa -in ${path}/key_pkcs1.pem -pubout -out ${path}/key_pub.pem 2>/dev/null
fi

if [[ ! -f "${path}/csr.pem" ]]; then
  echo "no CSR found ($path/csr.pem)"
  renew_csr=1
else
  echo "validating CSR ($path/csr.pem)"
  openssl req -in ${path}/csr.pem -verify

  if [[ $? != 0 ]]; then
    echo "invalid CSR"
    renew_csr=1
  fi

  echo "validating CSR subject ($path/csr.pem)"
  csr_subject=`openssl req -in ${path}/csr.pem -noout -subject | cut -d"=" -f2-`

  if [[ "$dn" != "$csr_subject" ]]; then
    echo "invalid CSR subject"
    renew_csr=1
  fi
fi

if [[ ${renew_csr} == 1 ]]; then
  echo "creating a new CSR ($path/csr.pem)"
  openssl req -new -key ${path}/key_pkcs1.pem -out ${path}/csr.pem -subj "$dn"
fi

# always return the CSR we do not know if nevisAdmin 4 has one
echo "returning CSR to nevisAdmin 4 master"
cp ${path}/csr.pem /tmp/nevisAdmin4/csr/${name}.pem
